/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: PrintExons                                         *
*                                                              *
*   Formatted printing of predicted exons.                     *
***************************************************************/

#include "geneid.h"

extern int GFF;

/* Print a predicted exon without gene prediction */
void PrintExon(exonGFF *e, char Name[], char* s, dict* dAA)
{
  char sAux[MAXAA];
  char* rs;
  long p1, p2;
  int nAA;
  
  p1 = e->Acceptor->Position + e->offset1 - COFFSET;
  p2 = e->Donor->Position + e->offset2 - COFFSET;
  
  
  /* Translation of nucleotids of exon to aminoacids */
  if (e->Strand == '+')
    /* Translate exon codons to aminoacids */
    nAA = Translate(p1,p2,
		    e->Frame,
		    (3 - e->Remainder)%3,
		    s, dAA, sAux);
  
  else
    {
      /* Reverse strand exon */
      if ((rs = (char*) calloc(p2-p1+2,sizeof(char))) == NULL)
	printError("Not enough space to translate reverse exon");
      
      ReverseSubSequence(p1, p2, s, rs);
      
      nAA = Translate(0,p2-p1,
		      e->Frame,
		      (3 - e->Remainder)%3,
		      rs, dAA, sAux);

      free(rs);
    }
  
  /* According to selected options print formatted */
  if (GFF)
    {
      /* GFF format */
      printf ("%s\t%s\t%s\t%ld\t%ld\t%5.3f\t%c\t%hd\n",
	      /* correct stop codon position, Terminal- & Terminal+ */ 
	      Name,
	      (e->evidence)? EVIDENCE : EXONS,
	      e->Type,
	      e->Acceptor->Position + e->offset1,
	      e->Donor->Position + e->offset2,
	      (e->Score==MAXSCORE)? 0.0: e->Score,
	      e->Strand,
	      e->Frame);
    }
  else
    {
      /* Default format */
      printf ("%8s %8ld %8ld\t%5.3f\t%c %hd %hd\t%5.3f\t%5.3f\t%5.3f\t%d\t%s\n",
	      /* correct stop codon position, Terminal- & Terminal+ */ 
	      e->Type,
	      e->Acceptor->Position + e->offset1,
	      e->Donor->Position + e->offset2,
	      (e->Score==MAXSCORE)? 0.0:e->Score,
	      e->Strand,
	      e->Frame,
	      (3 - e->Remainder)%3,
	      e->Acceptor->Score,
	      e->Donor->Score,
	      e->PartialScore,
	      nAA,
	      sAux);
      
    }
}

/* Print a group of exons of the same type */
void PrintExons (exonGFF *e, long ne, int type, char Name[],
		 long l1, long l2, char* Sequence,  dict* dAA) 
{
  long i;
  char Type[MAXTYPE];
  char strand;
  
  strand = (ne>0)? e->Strand : 'x'; 
  
  Type[0] = '\0';
  switch(type)
    {
    case FIRST: strcpy(Type,sFIRST);
       break;
    case INTERNAL:strcpy(Type,sINTERNAL);
      break;
    case TERMINAL:strcpy(Type,sTERMINAL);
      break;
    case TGA_GT:strcpy(Type,sTGA_GT);
      break;
    case TGA_STOP:strcpy(Type,sTGA_STOP);
      break;      
    default: strcpy(Type,"Exon");
      strand = 'x'; 
      break;
    }

  printf("# %ss(%c) predicted in sequence %s: [%ld,%ld]\n", 
	 Type,
	 strand,
	 Name, l1, l2);

  for (i=0;i<ne;i++)
    PrintExon((e+i), Name, Sequence, dAA);   
}


/* Print the exon of a gene */
void PrintGExon(exonGFF *e, char Name[], char* s, dict* dAA, 
	       long ngen, int AA1, int AA2, int nAA) 
{
  if (GFF)
    {
      /* GFF format */
      printf ("%s\t%s\t%s\t%ld\t%ld\t%5.3f\t%c\t%hd\tgene_%ld\t# AA-%d\tAA-%d\n",
	      /* correct stop codon position, Terminal- & Terminal+ */ 
	      Name,
         (e->evidence)? EVIDENCE : VERSION,     
	      e->Type,
	      e->Acceptor->Position + e->offset1,
	      e->Donor->Position + e->offset2,
	      (e->Score==MAXSCORE)? 0.0:e->Score,
	      e->Strand,
	      e->Frame,
	      ngen,
	      (e->Strand=='+')? nAA-AA2 : AA1,
	      (e->Strand=='+')? nAA-AA1 : AA2);
    }
  else
    {
      /* Default format for genes */
      printf ("%8s %8ld %8ld\t%5.3f\t%c %hd %hd\t%5.3f\t%5.3f\t%5.3f\tAA-%d AA-%d\tgene_%ld\n",
	      /* correct stop codon position, Terminal- & Terminal+ */ 
	      e->Type,
	      e->Acceptor->Position + e->offset1,
	      e->Donor->Position + e->offset2,
	      (e->Score==MAXSCORE)? 0.0:e->Score,
	      e->Strand,
	      e->Frame,
	      (3 - e->Remainder)%3,
	      e->Acceptor->Score,
	      e->Donor->Score,
	      e->PartialScore,
	      (e->Strand=='+')? nAA-AA2 : AA1,
	      (e->Strand=='+')? nAA-AA1 : AA2,
	      ngen);
    }
}
